FFmpeg
Loading...
Searching...
No Matches
softfloat.c
Go to the documentation of this file.
1/*
2 * copyright (c) 2006 Michael Niedermayer <michaelni@gmx.at>
3 *
4 * This file is part of FFmpeg.
5 *
6 * FFmpeg is free software; you can redistribute it and/or
7 * modify it under the terms of the GNU Lesser General Public
8 * License as published by the Free Software Foundation; either
9 * version 2.1 of the License, or (at your option) any later version.
10 *
11 * FFmpeg is distributed in the hope that it will be useful,
12 * but WITHOUT ANY WARRANTY; without even the implied warranty of
13 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
14 * Lesser General Public License for more details.
15 *
16 * You should have received a copy of the GNU Lesser General Public
17 * License along with FFmpeg; if not, write to the Free Software
18 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
19 */
20
21#include "libavutil/timer.h"
22
23#include <inttypes.h>
24
25#include "libavutil/softfloat.h"
26#include "libavutil/common.h"
27#include "libavutil/log.h"
28
29#include <stdio.h>
30
31static const SoftFloat FLOAT_0_017776489257 = {0x1234, 12};
32static const SoftFloat FLOAT_1374_40625 = {0xabcd, 25};
33static const SoftFloat FLOAT_0_1249694824218 = {0xFFF, 15};
34
35
36int main(void){
37 SoftFloat one= av_int2sf(1, 0);
38 SoftFloat sf1, sf2, sf3;
39 double d1, d2, d3;
40 int i, j;
42
43 d1= 1;
44 for(i= 0; i<10; i++){
45 d1= 1/(d1+1);
46 }
47 printf("test1 double=%d\n", (int)(d1 * (1<<24)));
48
49 sf1= one;
50 for(i= 0; i<10; i++){
51 sf1= av_div_sf(one, av_normalize_sf(av_add_sf(one, sf1)));
52 }
53 printf("test1 sf =%d\n", av_sf2int(sf1, 24));
54
55
56 for(i= 0; i<100; i++){
58 d1= i;
59 d2= i/100.0;
60 for(j= 0; j<1000; j++){
61 d1= (d1+1)*d2;
62 }
63 STOP_TIMER("float add mul")
64 }
65 printf("test2 double=%d\n", (int)(d1 * (1<<24)));
66
67 for(i= 0; i<100; i++){
69 sf1= av_int2sf(i, 0);
70 sf2= av_div_sf(av_int2sf(i, 2), av_int2sf(200, 3));
71 for(j= 0; j<1000; j++){
72 sf1= av_mul_sf(av_add_sf(sf1, one),sf2);
73 }
74 STOP_TIMER("softfloat add mul")
75 }
76 printf("test2 sf =%d (%d %d)\n", av_sf2int(sf1, 24), sf1.exp, sf1.mant);
77
78 d1 = 0.0177764893;
79 d2 = 1374.40625;
80 d3 = 0.1249694824;
81 d2 += d1;
82 d3 += d2;
83 printf("test3 double: %.10lf\n", d3);
84
86 sf2 = FLOAT_1374_40625;
88 sf2 = av_add_sf(sf1, sf2);
89 sf3 = av_add_sf(sf3, sf2);
90 printf("test3 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf3), sf3.mant, sf3.exp);
91
92 sf1 = av_int2sf(0xFFFFFFF0, 0);
93 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
94 sf1 = av_int2sf(0x00000010, 0);
95 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
96
97 sf1 = av_int2sf(0x1FFFFFFF, 0);
98 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
99 sf1 = av_int2sf(0xE0000001, 0);
100 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
101
102
103 sf1 = (SoftFloat){ 0x20000000, MIN_EXP };
104 sf1 = av_mul_sf(sf1, sf1);
105 printf("test5 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
106
107 sf1 = (SoftFloat){ 0x20000000, MIN_EXP };
108 sf2 = (SoftFloat){ 0x20000000, MAX_EXP };
109 i = av_cmp_sf(sf1, sf2);
110 j = av_cmp_sf(sf2, sf1);
111 sf1 = av_div_sf(sf1, sf2);
112 printf("test6 softfloat: %.10lf (0x%08x %d) %d %d\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp, i, j);
113
114 for(i= -50; i<50; i++) {
115 sf1= av_int2sf(i, 0);
116 for(j= -50; j<50; j++) {
117 int c;
118 sf2= av_int2sf(j, 0);
119 c = av_cmp_sf(sf1, sf2);
120 if (FFDIFFSIGN(i,j) != c && (FFDIFFSIGN(i,j)^c)<0) {
121 printf("av_cmp_sf failed at %d %d as %X\n", i, j, c);
122 }
123 c = av_gt_sf(sf1, sf2);
124 if ((i>j) != c) {
125 printf("av_gt_sf failed at %d %d as %X\n", i, j, c);
126 }
127 }
128 sf1 = av_int2sf(1, i);
129 for(j = -50; j < 50; j++) {
130 int c;
131 sf2 = av_int2sf(1, j);
132 c = av_cmp_sf(sf2, sf1);
133 if (FFDIFFSIGN(i,j) != c && (FFDIFFSIGN(i,j)^c) < 0) {
134 printf("av_cmp_sf failed2 at %d %d as %X\n", i, j, c);
135 }
136 c = av_gt_sf(sf1, sf2);
137 if ((i<j) != c) {
138 printf("av_gt_sf failed2 at %d %d as %X\n", i, j, c);
139 }
140 }
141 }
142
143
144 for(i= 0; i<4*36; i++){
145 int s, c;
146 double errs, errc;
147
148 av_sincos_sf(i*(1ULL<<32)/36/4, &s, &c);
149 errs = (double)s/ (1<<30) - sin(i*M_PI/36);
150 errc = (double)c/ (1<<30) - cos(i*M_PI/36);
151 if (fabs(errs) > 0.000000004 || fabs(errc) >0.000000004) {
152 printf("sincos FAIL %d %f %f %f %f\n", i, (float)s/ (1<<30), (float)c/ (1<<30), sin(i*M_PI/36), cos(i*M_PI/36));
153 }
154
155 }
156 return 0;
157
158}
#define i(width, name, range_min, range_max)
Definition cbs_h264.c:63
#define s(width, name)
Definition cbs_vp9.c:198
common internal and external API header
__device__ int printf(const char *,...)
static __device__ float fabs(float a)
#define AV_LOG_DEBUG
Stuff which is only useful for libav* developers.
Definition log.h:231
void av_log_set_level(int level)
Set the log level.
Definition log.c:476
#define FFDIFFSIGN(x, y)
Comparator.
Definition macros.h:45
#define M_PI
Definition mathematics.h:67
static const SoftFloat FLOAT_0_1249694824218
Definition softfloat.c:33
int main(void)
Definition softfloat.c:36
static const SoftFloat FLOAT_0_017776489257
Definition softfloat.c:31
static const SoftFloat FLOAT_1374_40625
Definition softfloat.c:32
static av_const SoftFloat av_add_sf(SoftFloat a, SoftFloat b)
Definition softfloat.h:162
static av_const int av_cmp_sf(SoftFloat a, SoftFloat b)
Compares two SoftFloats.
Definition softfloat.h:138
static av_const int av_sf2int(SoftFloat v, int frac_bits)
Converts a SoftFloat to an integer.
Definition softfloat.h:198
static av_const SoftFloat av_div_sf(SoftFloat a, SoftFloat b)
b has to be normalized and not zero.
Definition softfloat.h:116
static av_unused void av_sincos_sf(int a, int *s, int *c)
Rounding-to-nearest used.
Definition softfloat.h:244
#define MIN_EXP
Definition softfloat.h:30
static av_const SoftFloat av_mul_sf(SoftFloat a, SoftFloat b)
Definition softfloat.h:102
static av_const double av_sf2double(SoftFloat v)
Convert a SoftFloat to a double precision float.
Definition softfloat.h:52
static av_const SoftFloat av_int2sf(int v, int frac_bits)
Converts a mantisse and exponent to a SoftFloat.
Definition softfloat.h:185
#define MAX_EXP
Definition softfloat.h:31
static av_const SoftFloat av_normalize_sf(SoftFloat a)
Definition softfloat.h:57
static av_const int av_gt_sf(SoftFloat a, SoftFloat b)
Compares two SoftFloats.
Definition softfloat.h:150
int32_t mant
Definition softfloat.h:35
int32_t exp
Definition softfloat.h:36
high precision timer, useful to profile code
#define START_TIMER
Definition timer.h:166
#define STOP_TIMER(id)
Definition timer.h:167
static double c[64]